untyped

global function OnWeaponPrimaryAttack_weapon_softball
global function OnProjectileCollision_weapon_softball

#if SERVER
global function OnWeaponNpcPrimaryAttack_weapon_softball
#endif // #if SERVER

const FUSE_TIME = 0.5 //Applies once the grenade has stuck to a surface.

var function OnWeaponPrimaryAttack_weapon_softball( entity weapon, WeaponPrimaryAttackParams attackParams )
{
	entity player = weapon.GetWeaponOwner()

	weapon.EmitWeaponNpcSound( LOUD_WEAPON_AI_SOUND_RADIUS_MP, 0.2 )
	//vector bulletVec = ApplyVectorSpread( attackParams.dir, player.GetAttackSpreadAngle() * 2.0 )
	//attackParams.dir = bulletVec

	if ( IsServer() || weapon.ShouldPredictProjectiles() )
	{
		vector offset = Vector( 30.0, 6.0, -4.0 )
		if ( weapon.IsWeaponInAds() )
			offset = Vector( 30.0, 0.0, -3.0 )
		vector attackPos = player.OffsetPositionFromView( attackParams[ "pos" ], offset )	// forward, right, up
		
		//Triple Threat ammo
		if( weapon.HasMod( "triplethreat_softball" ) )
		{
			#if SERVER
			return FireTripleThreat_Softball( weapon, attackParams, true )
			#endif
		}
		//Emp Grenade ammo
		if( weapon.HasMod( "emp_softball" ) )
		{
			#if SERVER
			return FireGrenade( weapon, attackParams )
			#endif
		}
		else
			FireGrenade( weapon, attackParams )
	}
}

#if SERVER
var function OnWeaponNpcPrimaryAttack_weapon_softball( entity weapon, WeaponPrimaryAttackParams attackParams )
{
	weapon.EmitWeaponNpcSound( LOUD_WEAPON_AI_SOUND_RADIUS_MP, 0.2 )
	if( weapon.HasMod( "triplethreat_softball" ) )
		return FireTripleThreat_Softball( weapon, attackParams, false )
	else
		FireGrenade( weapon, attackParams, true )
}
#endif // #if SERVER

function FireGrenade( entity weapon, WeaponPrimaryAttackParams attackParams, isNPCFiring = false )
{
	vector angularVelocity = Vector( RandomFloatRange( -1200, 1200 ), 100, 0 )

	int damageType = DF_RAGDOLL | DF_EXPLOSION // | DF_GIB

	entity nade = weapon.FireWeaponGrenade( attackParams.pos, attackParams.dir, angularVelocity, 0.0 , damageType, damageType, !isNPCFiring, true, false )

	if ( nade )
	{
		#if SERVER
			Grenade_Init( nade, weapon )
		#else
			entity weaponOwner = weapon.GetWeaponOwner()
			SetTeam( nade, weaponOwner.GetTeam() )
		#endif
		if( weapon.HasMod( "sonar_softball" ) )
		{
			nade.SetModel( $"models/weapons/kunai/w_kunai_projectile.mdl" )
			return
		}
		if( weapon.HasMod( "thermite_softball" ) )
		{
			nade.SetModel( $"models/weapons/shuriken/w_shuriken.mdl" )
			return
		}
		if( weapon.HasMod( "northstar_softball" ) )
		{
			nade.SetModel( $"models/titans/light/titan_light_raptor.mdl" )
			return
		}
		if( weapon.HasMod( "cluster_softball" ) )
		{
			nade.SetModel( $"models/weapons/bullets/projectile_rocket_largest.mdl" )
			return
		}
		if( weapon.HasMod( "arcball_softball" ) )
		{
			nade.SetModel( $"models/dev/empty_model.mdl" )
			return	
		}
		if( weapon.HasMod( "smoke_softball" ) )
		{
			nade.SetModel( $"models/weapons/grenades/smoke_grenade_projectile.mdl" )
			return
		}
		if( weapon.HasMod( "gravity_softball" ) )
		{
			nade.SetModel( $"models/weapons/shuriken/w_shuriken.mdl" )
			return
		}
		if( weapon.HasMod( "emp_softball" ) )
		{
			nade.SetModel( $"models/weapons/grenades/arc_grenade_projectile.mdl" )
			return
		}
		if( weapon.HasMod( "error_softball" ) )
		{
			nade.SetModel( $"models/error.mdl" )
			return
		}
		else
		{
			EmitSoundOnEntity( nade, "Weapon_softball_Grenade_Emitter" )
		}
		
	}
}

void function OnProjectileCollision_weapon_softball( entity projectile, vector pos, vector normal, entity hitEnt, int hitbox, bool isCritical )
{
	//Use "projectile_ricochet_max_count" as a mark of hopup index
	//Default is 0, so hopups start from 1
	int hopupindex = projectile.GetProjectileWeaponSettingInt( eWeaponVar.projectile_ricochet_max_count )

	//Prevent triplethreat from being sticky, 8 is it's index
	if( !(hopupindex == 8) )
	{
		bool didStick = PlantSuperStickyGrenade( projectile, pos, normal, hitEnt, hitbox )
		if ( !didStick )
			return
	}

	#if SERVER
	entity player = projectile.GetOwner()

	if ( IsAlive( hitEnt ) && hitEnt.IsPlayer() )
	{
		EmitSoundOnEntityOnlyToPlayer( projectile, hitEnt, "weapon_softball_grenade_attached_1P" )
		EmitSoundOnEntityExceptToPlayer( projectile, hitEnt, "weapon_softball_grenade_attached_3P" )
	}
	else
	{
		EmitSoundOnEntity( projectile, "weapon_softball_grenade_attached_3P" )
	}

	//Default ammo
	if( hopupindex == 0 )
	{
		thread DetonateStickyAfterTime( projectile, FUSE_TIME, normal )
		return
	}
	//Thermite ammo
	if( hopupindex == 1 )
	{
		thread ThermiteBurn_Softball( 2.0, player, projectile )
		return
	}
	//Sonar ammo
	if( hopupindex == 2 )
	{
		thread SonarGrenadeThink_Softball( projectile )
		thread DestroySoftballProjectile( projectile, 1.0 )
		return
	}
	//Northstar ammo
	if( hopupindex == 3 )
	{
		thread NuclearCoreExplosion_Softball( pos, player, projectile )
		return
	}
	//Cluster ammo
	if( hopupindex == 4 )
	{
		ClusterRocket_Detonate_Softball( projectile, normal )
		return
	}
	//Arcball ammo
	if( hopupindex == 5 )
	{
		thread DetonateStickyAfterTime( projectile, 0.0, normal )
		return	
	}
	//Smoke ammo
	if( hopupindex == 6 )
	{
		ElectricGrenadeSmokescreen_Softball( projectile, FX_ELECTRIC_SMOKESCREEN_PILOT_AIR )
		thread DestroySoftballProjectile( projectile, 4.0)
		return
	}
	//Gravity ammo
	if( hopupindex == 7 )
	{
		thread GravityGrenadeThink_Softball( projectile, hitEnt, normal, pos )
		return
	}
	//Emp Grenade ammo
	if( hopupindex == 9 )
	{
		thread DetonateStickyAfterTime( projectile, 0.75, normal )
		return
	}
	if( hopupindex == 10 )
	{
		DetonateRandomGrenade( projectile, pos, normal, hitEnt, hitbox, isCritical )
		return
	}
	#endif
}

void function DetonateRandomGrenade( entity projectile, vector pos, vector normal, entity hitEnt, int hitbox, bool isCritical )
{
	#if SERVER
	entity player = projectile.GetOwner()
	switch( RandomInt(10) )
	{
		case 0:
			thread ThermiteBurn_Softball( 2.0, player, projectile )
			return
		case 1:
			thread ThermiteBurn_Softball( 2.0, player, projectile )
			return
		case 2:
			thread ThermiteBurn_Softball( 2.0, player, projectile )
			return
		case 3:
			thread SonarGrenadeThink_Softball( projectile )
			thread DestroySoftballProjectile( projectile, 1.0 )
			return
		case 4:
			ElectricGrenadeSmokescreen_Softball( projectile, FX_ELECTRIC_SMOKESCREEN_PILOT_AIR )
			thread DestroySoftballProjectile( projectile, 4.0)
			return
		case 5:
			thread DetonateStickyAfterTime( projectile, 0.0, normal )
			return
		case 6:
			thread DetonateStickyAfterTime( projectile, FUSE_TIME, normal )
			return
		case 7:
			ClusterRocket_Detonate_Softball( projectile, normal )
			return
		case 8:
			thread GravityGrenadeThink_Softball( projectile, hitEnt, normal, pos )
			return
		case 9:
			thread NuclearCoreExplosion_Softball( pos, player, projectile )
			return
		default:
			return
	}
	#endif
}

#if SERVER
// need this so grenade can use the normal to explode
void function DetonateStickyAfterTime( entity projectile, float delay, vector normal )
{
	wait delay
	if ( IsValid( projectile ) )
		projectile.GrenadeExplode( normal )
}

void function DestroySoftballProjectile( entity projectile, float duration )
{
	wait duration
	if ( IsValid( projectile ) )
		projectile.Destroy()
}
#endif